from pingpong import PingPong
from pingpong.gradio import GradioAlpacaChatPPManager
from pingpong.context import CtxAutoSummaryStrategy
from pingpong.context import CtxLastWindowStrategy


class TestStrategy():
	def test_last_window_strategy(self):
		pp_manager = GradioAlpacaChatPPManager()
		strategy = CtxLastWindowStrategy(2)

		for i in range(1):
			pp = PingPong(f"ping-{i}", f"pong-{i}")
			pp_manager.add_pingpong(pp)

		answers = f"""### Instruction: ping-0

### Response: pong-0"""
		assert strategy(pp_manager) == answers

		pp_manager = GradioAlpacaChatPPManager()
		for i in range(3):
			pp = PingPong(f"ping-{i}", f"pong-{i}")
			pp_manager.add_pingpong(pp)

		answers = f"""### Instruction: ping-1

### Response: pong-1

### Instruction: ping-2

### Response: pong-2"""
		assert strategy(pp_manager) == answers		


	def test_auto_summary_strategy(self):
		pp_manager = GradioAlpacaChatPPManager()
		strategy = CtxAutoSummaryStrategy(2)

		pp = PingPong("hello", "world")
		pp_manager.add_pingpong(pp)

		sum_req, to_summarize = strategy(pp_manager)
		assert sum_req == False

		pp = PingPong("hello2", "world2")
		pp_manager.add_pingpong(pp)

		sum_req, to_summarize = strategy(pp_manager)
		to_summarize_answer = """### Instruction: hello

### Response: world

### Instruction: hello2

### Response: world2"""
		assert sum_req == True
		assert to_summarize == to_summarize_answer
		assert strategy.last_idx == 2

		pp = PingPong("hello3", "world3")
		pp_manager.add_pingpong(pp)

		sum_req, to_summarize = strategy(pp_manager)
		assert sum_req == False

		pp = PingPong("hello4", "world4")
		pp_manager.add_pingpong(pp)

		sum_req, to_summarize = strategy(pp_manager)
		to_summarize_answer = """### Instruction: hello3

### Response: world3

### Instruction: hello4

### Response: world4"""
		assert sum_req == True
		assert to_summarize == to_summarize_answer
		assert strategy.last_idx == 4



from pingpong.pingpong import PromptFmt, UIFmt
from pingpong.alpaca import AlpacaPromptFmt, AlpacaChatPPManager

class GradioChatUIFmt(UIFmt):
  @classmethod
  def ui(cls, pingpong):
    return (pingpong.ping, pingpong.pong)

class GradioAlpacaChatPPManager(AlpacaChatPPManager):
  def build_uis(self, from_idx: int=0, fmt: UIFmt=GradioChatUIFmt):
    results = []

    for pingpong in self.pingpongs[from_idx:]:
      results.append(fmt.ui(pingpong))

    return results


from pingpong.pingpong import PPManager
from pingpong.pingpong import PromptFmt

class AlpacaPromptFmt(PromptFmt):
  @classmethod
  def ctx(cls, context):
    if context is None or context == "":
      return ""
    else:
      return f"""### Input: {context}

"""

  @classmethod
  def prompt(cls, pingpong):
    return f"""### Instruction: {pingpong.ping}

### Response: {pingpong.pong}"""

class AlpacaChatPPManager(PPManager):
  def build_prompts(self, from_idx: int=0, fmt: PromptFmt=AlpacaPromptFmt):
    results = fmt.ctx(self.ctx)

    for idx, pingpong in enumerate(self.pingpongs[from_idx:]):
      print(idx)
      results += fmt.prompt(pingpong)

      if idx != len(self.pingpongs[from_idx:])-1:
        results += """

"""

    return results


from setuptools import setup, find_packages

setup(
    name='bingbong',
    version='0.1',
    description='Ping pong management library for LLM applied application',
    author='chansung park',
    author_email='deep.diver.csp@gmail.com',
    url='https://github.com/deep-diver/PingPong',
    install_requires=[],
    packages=['pingpong'],
    package_dir={'':'src'},
    keywords=['LLM', 'pingpong', 'prompt', 'context', 'management'],
    python_reuqires='>=3.8',
    package_data={},
    zip_safe=False,
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11'
    ]
)


class PingPong:
  def __init__(self, ping, pong):
    self.ping = ping
    self.pong = pong

class PromptFmt:
  @classmethod
  def ctx(cls, context):
    pass

  @classmethod
  def prompt(cls, pingpong):
    pass

class UIFmt:
  @classmethod
  def ui(cls, pingpong):
    pass

class PPManager:
  def __init__(self, ctx: str=""):
    self.ctx = ctx
    self.pingpongs = []

  def add_pingpong(self, pingpong):
    self.pingpongs.append(pingpong)

  def pop_pingpong(self):
    return self.pingpongs.pop()

  def build_prompts(self, from_idx: int, fmt: PromptFmt):
    pass

  def build_uis(self, from_idx: int, fmt: UIFmt):
    pass
    


from pingpong import PingPong
from pingpong.gradio import GradioAlpacaChatPPManager

class TestPingpong():
    def test_ctx_alpaca_pingpong(self):
        pp = PingPong("hello", "world")
        pp_manager = GradioAlpacaChatPPManager()
        pp_manager.add_pingpong(pp)

        prompts = pp_manager.build_prompts()
        answers = f"""### Instruction: hello

### Response: world"""
        assert prompts == answers

        uis = pp_manager.build_uis()
        answers = [("hello", "world")]
        assert uis == answers

        pp_manager.ctx = "this is context"
        answers = f"""### Input: this is context

### Instruction: hello

### Response: world"""
        prompts = pp_manager.build_prompts()
        assert prompts == answers

    def test_single_gradio_alpaca_pingpong(self):
        pp = PingPong("hello", "world")
        pp_manager = GradioAlpacaChatPPManager()
        pp_manager.add_pingpong(pp)

        prompts = pp_manager.build_prompts()
        answers = f"""### Instruction: hello

### Response: world"""
        assert prompts == answers

        uis = pp_manager.build_uis()
        answers = [("hello", "world")]
        assert uis == answers

    def test_multi_gradio_pingpong(self):
        pp_manager = GradioAlpacaChatPPManager()

        for i in range(2):
            pp = PingPong(f"ping-{i}", f"pong-{i}")
            pp_manager.add_pingpong(pp)

        prompts = pp_manager.build_prompts()
        answers = f"""### Instruction: ping-0

### Response: pong-0

### Instruction: ping-1

### Response: pong-1"""
        assert prompts == answers

        uis = pp_manager.build_uis()
        answers = [("ping-0", "pong-0"), ("ping-1", "pong-1")]
        assert uis == answers
            